@@ -140,7 +140,7 @@ gpgme_error_to_errno (gpgme_error_t err)
       errno = EINVAL;
       return -1;
     case GPG_ERR_NOT_SUPPORTED:
-      errno = EOPNOTSUPP;
+      errno = ENOSYS;
       return -1;
     default:
       /* FIXME: Yeah, well.  */
