@@ -1,22 +1,23 @@
 /* t-keylist-sig.c - Regression test.
    Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2003 g10 Code GmbH
+   Copyright (C) 2001, 2003, 2004 g10 Code GmbH
 
    This file is part of GPGME.
  
    GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- 
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
    GPGME is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
- 
-   You should have received a copy of the GNU General Public License
-   along with GPGME; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 /* We need to include config.h so that we know whether we are building
    with large file system (LFS) support. */
@@ -309,10 +310,9 @@ main (int argc, char **argv)
 		   key->subkeys->next->keyid);
 	  exit (1);
 	}
-      if (key->subkeys->next->fpr)
+      if (!key->subkeys->next->fpr)
 	{
-	  fprintf (stderr, "Secondary key has unexpectedly a fingerprint: %s\n",
-		   key->subkeys->next->fpr);
+	  fprintf (stderr, "Secondary key has unexpectedly no fingerprint\n");
 	  exit (1);
 	}
       if (key->subkeys->next->expires)
@@ -466,7 +466,7 @@ main (int argc, char **argv)
         after importing the secret key.  We disable this test for
         now. */
 #ifdef __GNUC__
-#warning test disabled due to problems with gpg 1.3.4
+#warning test disabled due to problems with gpg 1.3.4 generated key
 #endif
       if (key->uids && (!key->uids->next->signatures /*|| key->uids->next->signatures->next*/))
 	{
