@@ -1,22 +1,23 @@
 /* t-keylist.c  - regression test
    Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2003 g10 Code GmbH
+   Copyright (C) 2001, 2003, 2004 g10 Code GmbH
 
    This file is part of GPGME.
  
    GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- 
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
    GPGME is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
- 
-   You should have received a copy of the GNU General Public License
-   along with GPGME; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 /* We need to include config.h so that we know whether we are building
    with large file system (LFS) support. */
@@ -43,6 +44,7 @@ struct
   char *issuer_name;
   char *chain_id;
   char *uid;
+  char *email;
   gpgme_validity_t validity;
   unsigned int key_length;
 }
@@ -52,7 +54,7 @@ keys[] =
       "CN=test cert 1,OU=Aegypten Project,O=g10 Code GmbH,L=D\xc3\xbcsseldorf,C=DE",
       "3CF405464F66ED4A7DF45BBDD1E4282E33BDB76E",
       "CN=test cert 1,OU=Aegypten Project,O=g10 Code GmbH,L=D\xc3\xbcsseldorf,C=DE",
-      GPGME_VALIDITY_ULTIMATE, 1024
+      NULL, GPGME_VALIDITY_ULTIMATE, 1024
     },
     { "DFA56FB5FC41E3A8921F77AD1622EEFD9152A5AD", 0, 909684190, 1009821790, "01",
       "1.2.840.113549.1.9.1=#63657274696679407063612E64666E2E6465,"
@@ -62,7 +64,7 @@ keys[] =
       "1.2.840.113549.1.9.1=#63657274696679407063612E64666E2E6465,"
       "CN=DFN Top Level Certification Authority,OU=DFN-PCA,"
       "O=Deutsches Forschungsnetz,C=DE",
-      GPGME_VALIDITY_NEVER, 2048
+      "<certify@pca.dfn.de>", GPGME_VALIDITY_NEVER, 2048
     },
     { "2C8F3C356AB761CB3674835B792CDA52937F9285", 0, 973183644, 1009735200, "15",
       "1.2.840.113549.1.9.1=#63657274696679407063612E64666E2E6465,"
@@ -72,7 +74,7 @@ keys[] =
       "1.2.840.113549.1.9.1=#63657274696679407063612E64666E2E6465,"
       "CN=DFN Server Certification Authority,OU=DFN-PCA,"
       "O=Deutsches Forschungsnetz,C=DE",
-      GPGME_VALIDITY_UNKNOWN, 2048
+      "<certify@pca.dfn.de>", GPGME_VALIDITY_UNKNOWN, 2048
     },
     { NULL }
   };
@@ -243,9 +245,10 @@ main (int argc, char **argv)
 	  fprintf (stderr, "Primary key unexpectedly unusable for certifications\n");
 	  exit (1);
 	}
-      if (key->subkeys->secret)
+      if (key->subkeys->secret != keys[i].secret)
 	{
-	  fprintf (stderr, "Primary key unexpectedly secret\n");
+	  fprintf (stderr, "Primary Key unexpectedly%s secret\n",
+		   key->secret ? "" : " not");
 	  exit (1);
 	}
       if (key->subkeys->pubkey_algo != GPGME_PK_RSA)
@@ -285,7 +288,8 @@ main (int argc, char **argv)
 	  exit (1);
 	}
 
-      if (!key->uids || key->uids->next)
+      /* Be tolerant against a missing email (ie, older gpgsm versions).  */
+      if (!key->uids || (key->uids->next && !keys[i].email))
 	{
 	  fprintf (stderr, "Key has unexpected number of user IDs\n");
 	  exit (1);
@@ -335,6 +339,20 @@ main (int argc, char **argv)
 		   key->uids->uid);
 	  exit (1);
 	}
+      if (key->uids->next && strcmp (key->uids->next->uid, keys[i].email))
+	{
+	  fprintf (stderr, "Unexpected email in user ID: %s\n",
+		   key->uids->next->uid);
+	  exit (1);
+	}
+      if (key->uids->next && strcmp (key->uids->next->uid, keys[i].email))
+	{
+	  fprintf (stderr, "Unexpected email in user ID: %s\n",
+		   key->uids->next->uid);
+	  exit (1);
+	}
+
+
 
       gpgme_key_unref (key);
       i++;
